# coding: utf-8
from inspect_report.models import Tasks, CheckSession, RulesStat, SeatStat, ScoreStat
from config.config import TABLE_PRE
import json
import logging
import kronos
from datetime import datetime, timedelta
"""定时任务

参考：https://github.com/jgorset/django-kronos
"""
logger = logging.getLogger('app_file')
score_item_zero = {'违规则零分': 100, '敏感词': 100}
score_item_service = {'异议处理': 10, '业务促成': 15, '预约及流转': 15, '服务介绍': 5, '6_引导流程_2个场景': 0, '未提供报价': 5,
                      '未做优惠费率介绍': 5}
score_item_business = {'结束语': 5, '服务态度': 10, '服务用语': 10, '语速识别': 5, '静音识别': 5, '6_2_引导流程_投诉安抚': 0,
                       '开场白': 10}


@kronos.register('0 1 * * *')
def rule_stat(start_date=None, end_date=None):
    """
    首页概述-违规项统计
    :return:
    """
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[rule_stat]rule stat start...')
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if RulesStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id'], 'violationRuleCount__gt': 0}
        checks = tn.objects.filter(**session_condition).values('id', 'checkResult', 'agentName', 'customName', 'score',
                                                               'remainTime', 'taskId', 'sessionId')
        rules_list = []
        for check in checks:
            result = check['checkResult']
            if result:
                data = json.loads(result)
                for d in data:
                    if 'isViolation' in d.keys() and d['isViolation'] and 'rule' in d.keys() and 'name' in d['rule'].keys():
                        name = d['rule']['name']
                        rule_obj = {'create_date': start_date, 'sessionCollectionId': t['sessionCollectionId'],
                                    'rule': name, 'rule_num': 1, 'task': t['name'].split('_')[0],
                                    'agentName': check['agentName'], 'customName': check['customName'],
                                    'score': check['score'], 'remainTime': check['remainTime'],
                                    'taskId': check['taskId'], 'sessionId': check['sessionId'],
                                    'session_table_id': check['id']}
                        rules_list.append(RulesStat(**rule_obj))
                        stat_count += 1
        RulesStat.objects.bulk_create(rules_list)
    logger.info('[rule_stat]rule stat num: [%s]', stat_count)
    logger.info('[rule_stat]rule stat end.')


@kronos.register('0 1 * * *')
def seat_stat(start_date=None, end_date=None):
    """
    首页概述-违规坐席统计
    :return:
    """
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[seat_stat]violate seat stat start...')
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if SeatStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'taskId', 'violationRuleCount')
        seat_dict = {}
        seat_list = []
        for check in checks:
            stat_count += 1
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
            else:
                seat_dict[check['agentName']] = {'create_date': start_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'], 'task': t['name'].split('_')[0],
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(SeatStat(**seat_obj))
        SeatStat.objects.bulk_create(seat_list)
    logger.info('[seat_stat]violate seat count: [%s]', stat_count)
    logger.info('[seat_stat]violate seat stat end.')


@kronos.register('0 1 * * *')
def score_stat(start_date=None, end_date=None):
    """
    坐席得分统计
    :return:
    """
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[score_stat]seat score stat start...')
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if ScoreStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'taskId', 'violationRuleCount',
                                                               'startTime', 'score', 'scoreItemRecord')
        seat_dict = {}
        seat_list = []
        for check in checks:
            stat_count += 1
            score_item = json.loads(check['scoreItemRecord'])
            service_score = 55
            business_score = 45
            validate_zero = 0

            for m in score_item:
                if m['scoreItemName'] in score_item_service.keys():
                    service_score -= m['score']
                if m['scoreItemName'] in score_item_business.keys():
                    business_score -= m['score']
                if m['scoreItemName'] in score_item_zero.keys():
                    validate_zero += 1
            if validate_zero > 0:
                score = 0
            else:
                score = service_score + business_score
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
                seat['validate_num'] += len(score_item)
                seat['score'] = round((seat['score'] + score)/2, 2)
                seat['service_score'] = round((seat['service_score'] + service_score)/2, 2)
                seat['business_score'] = round((seat['business_score'] + business_score)/2, 2)
            else:
                seat_dict[check['agentName']] = {'create_date': start_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'], 'task': t['name'].split('_')[0],
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1,
                                                 'score': score, 'service_score': service_score,
                                                 'business_score': business_score, 'validate_zero': validate_zero}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
                seat_dict[check['agentName']]['validate_num'] = len(score_item)
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(ScoreStat(**seat_obj))
        ScoreStat.objects.bulk_create(seat_list)
    logger.info('[score_stat]seat count: [%s]', stat_count)
    logger.info('[score_stat]seat score stat end.')
